from django.db import models
from django_smalluuid.models import SmallUUIDField, uuid_default, uuid_typed_default


class TestModel(models.Model):
    uuid = SmallUUIDField(default=uuid_default())


class TypedTestModel(models.Model):
    uuid = SmallUUIDField(default=uuid_typed_default(type=42))



from django import forms
from smalluuid.smalluuid import TypedSmallUUID
from django_smalluuid.forms import ShortUUIDField


class TestForm(forms.Form):
    uuid = ShortUUIDField()


class TypedTestForm(forms.Form):
    uuid = ShortUUIDField(uuid_class=TypedSmallUUID)



from django.core.exceptions import ValidationError
from django.forms import UUIDField
from django.utils import six
from django.utils.module_loading import import_string
from django.utils.translation import ugettext_lazy as _

from django_smalluuid import settings


class ShortUUIDField(UUIDField):
    default_error_messages = {
        'invalid': _('Enter a valid short-form UUID.'),
    }

    def __init__(self, uuid_class=settings.DEFAULT_CLASS, *args, **kwargs):
        self.uuid_class = uuid_class
        if isinstance(self.uuid_class, six.string_types):
            self.uuid_class = import_string(uuid_class)
        super(ShortUUIDField, self).__init__(*args, **kwargs)

    def prepare_value(self, value):
        if isinstance(value, self.uuid_class):
            return six.text_type(value)
        return value

    def to_python(self, value):
        if value in self.empty_values:
            return None
        if not isinstance(value, self.uuid_class):
            try:
                value = self.uuid_class(value)
            except ValueError:
                raise ValidationError(self.error_messages['invalid'], code='invalid')
        return value



from django.core.exceptions import ValidationError
from django.test import TestCase
from django.db import connection
from smalluuid import SmallUUID
from smalluuid import TypedSmallUUID

from django_smalluuid import models, forms
from django_smalluuid.forms import ShortUUIDField
from testapp.forms import TestForm, TypedTestForm
from testapp.models import TestModel, TypedTestModel


class ModelFieldTestCase(TestCase):

    def test_unique_by_default(self):
        field = models.SmallUUIDField()
        self.assertTrue(field.unique)

    def test_force_not_unique(self):
        field = models.SmallUUIDField(unique=False)
        self.assertFalse(field.unique)

    def test_to_python(self):
        field = models.SmallUUIDField()
        out = field.to_python('IBNApQOzTHGzdjkSt6t-Jg')
        self.assertIsInstance(out, SmallUUID)

    def test_to_python_invalid_value(self):
        field = models.SmallUUIDField()
        self.assertRaises(ValidationError, field.to_python, 'xxx')

    def test_to_python_custom_class(self):
        field = models.SmallUUIDField(uuid_class=TypedSmallUUID)
        out = field.to_python('IBNApQOzTHGzdjkSt6t-Jg')
        self.assertIsInstance(out, TypedSmallUUID)

    def test_to_python_class_string(self):
        field = models.SmallUUIDField(uuid_class='smalluuid.TypedSmallUUID')
        out = field.to_python('IBNApQOzTHGzdjkSt6t-Jg')
        self.assertIsInstance(out, TypedSmallUUID)

    def test_prep(self):
        field = models.SmallUUIDField()
        out = field.get_db_prep_value(SmallUUID('IBNApQOzTHGzdjkSt6t-Jg'), connection=connection)
        self.assertEqual(out, '201340a503b34c71b3763912b7ab7e26')

    def test_form_field(self):
        field = models.SmallUUIDField()
        self.assertIsInstance(field.formfield(), forms.ShortUUIDField)


class FormFieldTestCase(TestCase):

    def test_valid(self):
        form = TestForm({'uuid': 'IBNApQOzTHGzdjkSt6t-Jg'})
        self.assertTrue(form.is_valid(), form.errors)

    def test_invalid(self):
        form = TestForm({'uuid': 'xxx'})
        self.assertFalse(form.is_valid(), form.errors)

    def test_none(self):
        form = TestForm({'uuid': None})
        self.assertFalse(form.is_valid(), form.errors)

    def test_typed_valid(self):
        form = TypedTestForm({'uuid': 'IBNApQOzTHGzdjkSt6t-Jg'})
        self.assertTrue(form.is_valid(), form.errors)

    def test_typed_invalid(self):
        form = TypedTestForm({'uuid': 'xxx'})
        self.assertFalse(form.is_valid(), form.errors)

    def test_typed_none(self):
        form = TypedTestForm({'uuid': None})
        self.assertFalse(form.is_valid(), form.errors)

    def test_prepare_value(self):
        field = ShortUUIDField()
        self.assertEqual(field.prepare_value('IBNApQOzTHGzdjkSt6t-Jg'), 'IBNApQOzTHGzdjkSt6t-Jg')


class ModelTestCase(TestCase):

    def test_value_created(self):
        model = TestModel.objects.create()
        self.assertIsInstance(model.uuid, SmallUUID)
        self.assertEqual(model.uuid.version, 4)
        self.assertEqual(len(str(model.uuid)), 22)

    def test_persist_reload(self):
        model = TestModel.objects.create()
        model.save()
        uuid = TestModel.objects.values_list('uuid', flat=True)[0]
        self.assertEqual(model.uuid, uuid)

    def test_filter(self):
        TestModel.objects.create()
        model = TestModel.objects.create()
        TestModel.objects.create()

        uuid = TestModel.objects.filter(uuid=str(model.uuid)).values_list('uuid', flat=True)[0]
        self.assertIsInstance(model.uuid, SmallUUID)
        self.assertEqual(model.uuid, uuid)


class TypedModelTestCase(TestCase):

    def test_value_created(self):
        model =TypedTestModel.objects.create()
        self.assertIsInstance(model.uuid, TypedSmallUUID)
        self.assertEqual(model.uuid.version, 4)
        self.assertEqual(model.uuid.type, 42)
        self.assertEqual(len(str(model.uuid)), 22)

    def test_persist_reload(self):
        model = TypedTestModel.objects.create()
        model.save()
        uuid = TypedTestModel.objects.values_list('uuid', flat=True)[0]
        self.assertEqual(model.uuid, uuid)




#!/usr/bin/env python

from os.path import exists
from setuptools import setup, find_packages

from django_smalluuid import __version__

setup(
    name='django-smalluuid',
    version=__version__,
    author='Adam Charnock',
    author_email='adam@adamcharnock.com',
    packages=find_packages(),
    scripts=[],
    url='https://github.com/adamcharnock/django-smalluuid',
    license='MIT',
    description='Short-form UUID field for Django 1.8 and above',
    long_description=open('README.rst').read() if exists("README.rst") else "",
    install_requires=[
        'django>=1.8',
        'smalluuid>=0.1.4',
    ],
)



from django.core.exceptions import ValidationError
from django.db.models.fields import UUIDField
from django.utils import six
from django.utils.deconstruct import deconstructible
from django.utils.module_loading import import_string
from django.utils.translation import ugettext_lazy as _
import smalluuid

from django_smalluuid import settings, forms


class SmallUUIDField(UUIDField):
    default_error_messages = {
        'invalid': _("'%(value)s' is not a valid short-form UUID."),
    }
    description = 'Short-form universally unique identifier'

    def __init__(self, verbose_name=None, uuid_class=settings.DEFAULT_CLASS, **kwargs):
        self.uuid_class = uuid_class
        if isinstance(self.uuid_class, six.string_types):
            self.uuid_class = import_string(uuid_class)
        # unique=True is a much more sensible default for UUIDs
        kwargs.setdefault('unique', True)
        super(SmallUUIDField, self).__init__(verbose_name, **kwargs)

    def get_db_prep_value(self, value, *args, **kwargs):
        if isinstance(value, six.string_types):
            value = self.uuid_class(value)
        value = value.hex_grouped
        return super(SmallUUIDField, self).get_db_prep_value(value, *args, **kwargs)

    def to_python(self, value):
        if value and not isinstance(value, self.uuid_class):
            try:
                return self.uuid_class(value)
            except ValueError:
                raise ValidationError(
                    self.error_messages['invalid'],
                    code='invalid',
                    params={'value': value},
                )
        return value

    def formfield(self, **kwargs):
        defaults = {
            'form_class': forms.ShortUUIDField,
            'uuid_class': self.uuid_class,
        }
        defaults.update(kwargs)
        return super(SmallUUIDField, self).formfield(**defaults)

    def from_db_value(self, value, expression, connection, context):
        if not value:
            return None
        return self.uuid_class(int=value.int)


@deconstructible
class UUIDDefault(object):

    def __init__(self, version=None):
        self.version = version

    def __call__(self):
        return smalluuid.SmallUUID(version=self.version)

uuid_default = UUIDDefault


@deconstructible
class UUIDTypedDefault(object):

    def __init__(self, version=None, type=None):
        self.version = version
        self.type = type

    def __call__(self):
        return smalluuid.TypedSmallUUID(version=self.version, type=self.type)

uuid_typed_default = UUIDTypedDefault
